package de.frajul.endlessroll.main.screens;

import android.view.View;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.ToggleButton;

import de.frajul.endlessroll.R;
import de.frajul.endlessroll.main.GameActivity;
import de.frajul.endlessroll.views.ConfirmDialog;
import de.frajul.endlessroll.views.TopBar;

/**
 * Created by Julian on 10.09.2017.
 */

public class SettingsScreen extends Screen<LinearLayout> implements View.OnClickListener, ConfirmDialog.ConfirmDialogListener {

    private final int CALL_ID_CONFIRM_RESET = 1;

    private TopBar topBar;
    private ToggleButton musicToggle;
    private ToggleButton soundToggle;
    private Button resetButton;
    private boolean resetPressed = false;

    private ConfirmDialog resetConfirmDialog;

    public SettingsScreen(GameActivity gameActivity) {
        super(ScreenType.SETTINGS, gameActivity, R.layout.settings);
        topBar = super.createTopBar(R.id.settings_topbar);
        musicToggle = (ToggleButton) layout.findViewById(R.id.settings_musictoggle);
        musicToggle.setOnClickListener(this);
        soundToggle = (ToggleButton) layout.findViewById(R.id.settings_soundtoggle);
        soundToggle.setOnClickListener(this);
        resetButton = (Button) layout.findViewById(R.id.settings_reset);
        resetButton.setOnClickListener(this);

        resetConfirmDialog = new ConfirmDialog(gameActivity, this);
    }

    @Override
    public void prepareToBeShown() {
        gameActivity.getSoundManager().menuMusic.start();

        topBar.update();
        musicToggle.setChecked(!gameActivity.getSoundManager().isMusicMuted());
        soundToggle.setChecked(!gameActivity.getSoundManager().isSoundsMuted());
        resetButton.setEnabled(super.caller != ScreenType.GAME);
        resetPressed = false;
    }

    @Override
    public void onBackKeyDown() {
        gameActivity.getDataStorageHandler()
                .writeSoundsMuted(gameActivity.getSoundManager().isSoundsMuted());
        gameActivity.getDataStorageHandler()
                .writeMusicMuted(gameActivity.getSoundManager().isMusicMuted());
        //ResetButton is disabled if caller is GAME, because shortMenu or GoalMessage would still be visible on next level start
        if (resetPressed && (super.caller == ScreenType.LEVELS || super.caller == ScreenType.GAME)) {
            super.caller = ScreenType.WORLDS;
        }
        flipToCaller();
    }

    @Override
    public void onClick(View v) {
        if (v.equals(musicToggle)) {
            gameActivity.getSoundManager().setMusicMuted(!musicToggle.isChecked());
        } else if (v.equals(soundToggle)) {
            gameActivity.getSoundManager().setSoundsMuted(!soundToggle.isChecked());
        } else if (v.equals(resetButton)) {
            resetConfirmDialog.show(CALL_ID_CONFIRM_RESET, R.string.confirm_dialog_reset);
        }
    }

    @Override
    public void onConfirmed(int callId) {
        if (callId == CALL_ID_CONFIRM_RESET) {
            gameActivity.resetData();
            prepareToBeShown();
            resetPressed = true;
        }
    }
}
